#!/usr/bin/env node

const program = require('commander');
const path = require('path');
const fs = require('fs');
const rimraf = require('rimraf');
const shell = require('shelljs');
const execa = require('execa');
const { resolve, isRootPath } = require('../lib/local-path');
const register = require('../lib/register');
const compiler = require('../lib/compiler');
const deserialize = require('../lib/deserialize');

process.env.NODO_ENV = 'production';

(async () => {
  try {
    await register();

    const config = deserialize(process.env.MPT_CONFIG);

    program.usage('[template]').option('-e, --exec <value>', '指定执行自定义构建命令').parse(process.argv);

    if (program.exec) {
      const args = program.exec.split(' ');
      execa(args.shift(), args, { stdio: 'inherit', srderr: 'inherit' });
    } else {
      rimraf.sync(resolve('./dist'));
      await compiler('prod', config);
    }

    if (isRootPath()) {
      await execa('yarn', ['workspaces', 'run', 'build'], { stdio: 'inherit', srderr: 'inherit' });

      const packagesPath = resolve('./micro-apps');
      const pkgFiles = fs.readdirSync(packagesPath);
      pkgFiles.forEach((pkgFile) => {
        const pkgPath = path.join(packagesPath, pkgFile);
        const stats = fs.statSync(pkgPath);
        if (stats.isDirectory()) {
          const subDir = fs.readdirSync(pkgPath);
          subDir.forEach((file) => {
            if (file === 'mopocket.config.js') {
              const distPath = resolve(`./dist/${pkgFile}`);
              fs.mkdirSync(distPath);
              shell.cp('-R', path.join(pkgPath, 'dist/*'), distPath);
            }
          });
        }
      });
    }
  } catch (error) {
    console.error(error);
  }
})();
