#!/usr/bin/env node

const program = require('commander');
const exists = require('fs').existsSync;
const path = require('path');
const chalk = require('chalk');
const logger = require('../lib/logger');
const generate = require('../lib/generate');
const localPath = require('../lib/local-path');

const getTemplatePath = localPath.getTemplatePath;

/**
 * Usage.
 */

program.usage('<template> [name]');

/**
 * Help.
 */

program.on('--help', () => {
  console.log('  例子:');
  console.log();
  console.log(chalk.gray('    # 基于模板创建微应用'));
  console.log('    $ mpt create react-antd my-app');
});

/**
 * Help.
 */

function help() {
  program.parse(process.argv);
  if (program.args.length < 1) return program.help();
}
help();

/**
 * Settings.
 */

const template = program.args[0];
const name = program.args[1];
const to = path.normalize(path.join(process.cwd(), `./packages/${name}`));

/**
 * Padding.
 */

process.on('exit', () => {
  console.log();
});

/**
 * Check, download and generate the project.
 */

function run() {
  const templatePath = getTemplatePath(template);
  if (exists(templatePath)) {
    generate(name, templatePath, to, (err) => {
      if (err) logger.fatal(err);
      logger.success('Generated "%s".', name);
    });
  } else {
    logger.fatal('Local template "%s" not found.', template);
  }
}

run();
