#!/usr/bin/env node

const program = require('commander');
const WebpackDevServer = require('webpack-dev-server');
const execa = require('execa');
const fs = require('fs');
const serialize = require('serialize-javascript');
const register = require('../lib/register');
const { resolve, isRootPath } = require('../lib/local-path');
const compiler = require('../lib/compiler');
const deserialize = require('../lib/deserialize');

process.env.NODO_ENV = 'development';

(async () => {
  try {
    await register();

    const config = deserialize(process.env.MPT_CONFIG);

    program.usage('[options]').option('-e, --exec <value>', '指定执行自定义构建命令').parse(process.argv);

    if (program.exec) {
      const args = program.exec.split(' ');
      execa(args.shift(), args, { stdio: 'inherit', srderr: 'inherit' });
    } else {
      const c = await compiler('dev', config);
      const server = new WebpackDevServer(c, config.devServer);
      server.listen(config.devServer.port, config.devServer.host);
    }

    if (isRootPath()) {
      const dir = fs.readdirSync(resolve('./micro-apps'));
      dir.forEach((workspace) => {
        if (fs.existsSync(resolve(`./micro-apps/${workspace}/package.json`))) {
          execa('yarn', ['workspace', workspace, 'run', 'dev'], { stdio: 'inherit', srderr: 'inherit' });
        }
      });
    }
  } catch (error) {
    console.error(error);
  }
})();
