#!/usr/bin/env node

const program = require('commander');
const exists = require('fs').existsSync;
const path = require('path');
const chalk = require('chalk');
const logger = require('../lib/logger');
const generate = require('../lib/generate');
const localPath = require('../lib/local-path');

const getTemplatePath = localPath.getTemplatePath;

/**
 * Usage.
 */

program.usage('<name>');

/**
 * Help.
 */

program.on('--help', () => {
  console.log('  例子:');
  console.log();
  console.log(chalk.gray('    # 初始化新建项目'));
  console.log('    $ mpt init my-app');
});

/**
 * Help.
 */

function help() {
  program.parse(process.argv);
  if (program.args.length < 1) return program.help();
}
help();

/**
 * Settings.
 */

const name = program.args[0];
const to = path.normalize(path.join(process.cwd(), `./${name}`));

/**
 * Check, download and generate the project.
 */

function run() {
  const templatePath = getTemplatePath('root');
  console.log(templatePath);
  if (exists(templatePath)) {
    generate(name, templatePath, to, (err) => {
      if (err) logger.fatal(err);
      logger.success('Generated "%s".', name);
    });
  } else {
    logger.fatal('Local template "%s" not found.', 'root');
  }
}

run();
