const path = require('path');
const htmlWebpackPlugin = require('html-webpack-plugin');
const webpack = require('webpack');
const { resolve, isRootPath, getPkgName } = require('../lib/local-path');

module.exports = {
  entry: {
    main: [resolve('./src/index')],
  },
  output: {
    filename: '[name].[hash:8].js',
    path: resolve('./dist'),
    publicPath: '/',
    library: `${getPkgName()}-[name]`,
    libraryTarget: 'umd',
    jsonpFunction: `webpackJsonp_${getPkgName()}`,
  },
  module: {
    rule: {
      babel: {
        test: /\.ts(x?)$/ || /\.js(x?)$/,
        exclude: /(node_modules|bower_components)/,
        use: {
          'babel-loader': {
            loader: require.resolve('babel-loader'),
            options: {
              exclude: ['node_modules'],
              cacheDirectory: true,
            },
          },
        },
      },
      file: {
        test: /\.(jpe?g|png|gif|ico|svg)$/,
        use: {
          'file-loader': {
            loader: require.resolve('file-loader'),
            options: {
              name: 'assets/[name].[hash:8].[ext]',
            },
          },
        },
      },
    },
  },
  plugin: {
    htmlWebpackPlugin: {
      plugin: htmlWebpackPlugin,
      args: [
        {
          template: resolve('./src/index.html'),
        },
      ],
    },
    definePlugin: {
      plugin: webpack.DefinePlugin,
      args: [
        {
          __MPT_CONFIG__: process.env.MPT_CONFIG,
          __MPT_REG_APPS__: process.env.MPT_REG_APPS,
          __MPT_REG_COMPS__: process.env.MPT_REG_COMPS,
        },
      ],
    },
  },
  resolve: {
    extensions: ['.ts', '.tsx', '.js', '.jsx', '.less', '.css'],
    alias: {
      '@micro-apps': resolve(`${isRootPath() ? './' : '../../'}micro-apps`),
      '@micro-components': resolve(`${isRootPath() ? './' : '../../'}micro-components`),
      '@master': resolve(`${isRootPath() ? './' : '../../'}src`),
      '@': resolve('./src'),
    },
  },
  optimization: {
    splitChunks: {
      chunks: 'all',
      maxAsyncRequests: 30,
      maxInitialRequests: 30,
      minSize: 20000,
    },
  },
};
