const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const OptimizeCssAssetsWebpackPlugin = require('optimize-css-assets-webpack-plugin');
const webpackBar = require('webpackbar');
const fs = require('fs');
const { resolve, getPkgPath, isRootPath, getPkgName } = require('../lib/local-path');

module.exports = {
  mode: 'production',
  devtool: 'hidden-source-map',
  output: {
    filename: '[name].[contenthash:8].js',
    path: resolve('./dist/static/'),
    publicPath: `${isRootPath() ? '' : getPkgPath()}/static/`,
  },
  module: {
    rule: {
      style: {
        test: /\.(less|css)$/,
        use: {
          miniCssExtractPlugin: { loader: MiniCssExtractPlugin.loader },
          'css-loader': {
            loader: require.resolve('css-loader'),
            options: {
              sourceMap: false,
              modules: {
                auto: true,
                localIdentName: '[local]__[hash:base64:8]',
                exportLocalsConvention: 'camelCase',
              },
            },
          },
          'postcss-loader': {
            loader: require.resolve('postcss-loader'),
          },
          'less-loader': {
            loader: require.resolve('less-loader'),
            options: {
              sourceMap: false,
              lessOptions: {
                javascriptEnabled: true,
              },
            },
          },
        },
      },
    },
  },
  plugin: {
    cleanWebpackPlugin: {
      plugin: CleanWebpackPlugin,
    },
    miniCssExtractPlugin: {
      plugin: MiniCssExtractPlugin,
      args: [
        {
          filename: '[name].[contenthash:8].css',
        },
      ],
    },
    htmlWebpackPlugin: {
      plugin: HtmlWebpackPlugin,
      args: [
        {
          template: resolve('./src/index.html'),
          filename: resolve('./dist/index.html'),
          favicon: fs.existsSync(resolve('./src/favicon.ico')) ? resolve('./src/favicon.ico') : null,
        },
      ],
    },
    hashedModuleIdsPlugin: {
      plugin: webpack.HashedModuleIdsPlugin,
    },
    optimizeCssAssetsWebpackPlugin: {
      plugin: OptimizeCssAssetsWebpackPlugin,
      args: [
        {
          cssProcessPluginOptions: {
            preset: ['default', { discardComments: { removeAll: true } }],
          },
        },
      ],
    },
    webpackBar: {
      plugin: webpackBar,
      args: [
        {
          name: getPkgName(),
        },
      ],
    },
  },
};
