const serialize = require('serialize-javascript');
const path = require('path');
const fs = require('fs');
const lodash = require('lodash');
const { resolve, isRootPath, getPkgName } = require('../lib/local-path');
const defaultConfig = require('../lib/default-config');
const deserialize = require('../lib/deserialize');

module.exports = async function () {
  let cfg = require(resolve('./mopocket.config'));
  await defaultConfig(cfg);
  const allConfig = [];

  if (!isRootPath()) {
    // 如果在根目录执行的命令，就从node主进程的env中拿取已经缓存的config，无需重复生成，
    if (process.env.MPT_ALL_CONFIG) {
      const allCfg = deserialize(process.env.MPT_ALL_CONFIG);
      cfg = allCfg.find((cfg) => cfg.name === getPkgName());
    } else {
      cfg = lodash.merge({}, require(resolve('../../mopocket.config.js')), cfg);
    }
  } else {
    allConfig.push(cfg); // 将root下的配置push
    // 自动生成微应用注册配置
    const regApps = [];
    const regComponents = [];

    const mapRegMicroPath = {
      apps: resolve('./micro-apps'),
      components: resolve('./micro-components'),
    };

    // 收集微应用文件路径
    const allMicroDirPath = [];

    Object.keys(mapRegMicroPath).forEach((key) => {
      const regMicroPath = mapRegMicroPath[key];
      if (!fs.existsSync(regMicroPath)) return;
      fs.readdirSync(regMicroPath)
        .filter((curPath) => {
          const stats = fs.statSync(path.join(regMicroPath, `./${curPath}`));
          return stats.isDirectory();
        })
        .forEach((curPath) => {
          allMicroDirPath.push(path.join(regMicroPath, `./${curPath}`));
        });
    });

    for (let i = 0; i < allMicroDirPath.length; i++) {
      const microDirPath = allMicroDirPath[i];
      const microName = path.basename(microDirPath);
      const microType = path.basename(path.join(microDirPath, '..'));

      const microDir = fs.readdirSync(microDirPath);
      const isMptCfg = microDir.find((file) => file === 'mopocket.config.js');
      if (isMptCfg) {
        let curAppCfg = require(path.join(microDirPath, './mopocket.config.js'));
        await defaultConfig(curAppCfg);
        curAppCfg = lodash.merge({ name: microName }, cfg, curAppCfg);
        allConfig.push(curAppCfg);

        if (microType === 'micro-apps') {
          regApps.push({
            name: microName,
            entry: process.env.WEBPACK_DEV_SERVER
              ? `//${curAppCfg.devServer.host}:${curAppCfg.devServer.port}`
              : `/${microName}`,
            container: curAppCfg.container,
            activeRule: `/app-${microName}`,
          });
        } else if (microType === 'micro-components') {
          regComponents.push({
            name: microName,
            entry: process.env.WEBPACK_DEV_SERVER
              ? `//${curAppCfg.devServer.host}:${curAppCfg.devServer.port}`
              : `/${microName}`,
          });
        }
      }
    }

    // register remote app
    if (cfg.remoteMicroApps) {
      Object.keys(cfg.remoteMicroApps).forEach((key) => {
        regApps.push({
          name: key,
          entry: `/${key}`,
          container: cfg.remoteMicroApps[key].container || '#root',
          activeRule: `/app-${key}`,
        });
      });
    }

    // register remote component
    if (cfg.remoteMicroComponents) {
      Object.keys(cfg.remoteMicroComponents).forEach((key) => {
        regComponents.push({
          name: key,
          entry: `/${key}`,
        });
      });
    }

    process.env.MPT_REG_APPS = serialize(regApps);
    process.env.MPT_REG_COMPS = serialize(regComponents);

    // 从根目录执行命令，会自动load所有子应用配置文件装载到node主进程的env中，避免重复load
    process.env.MPT_ALL_CONFIG = serialize(allConfig);
  }

  process.env.MPT_CONFIG = serialize(cfg);
};
